""" Abstract Storage. """
from __future__ import print_function

from os import path as op, walk


class BaseStorage():

    """ Base class for storages. """

    def __init__(self, collect, verbose=False):
        self.verbose = verbose
        self.collect = collect

    def __iter__(self):
        """ Seek static files and result full and relative paths.

        :return generator: Walk files

        """
        app_and_blueprints = self.collect.filter(
            [self.collect.app] + list(self.collect.blueprints.values()))
        for bp in app_and_blueprints:
            if bp.has_static_folder and op.isdir(bp.static_folder):
                for root, _, files in walk(bp.static_folder):
                    for f in files:
                        fpath = op.join(root, f)
                        opath = op.relpath(fpath, bp.static_folder.rstrip('/'))
                        if bp.static_url_path and self.collect.static_url and \
                                bp.static_url_path.startswith(
                                    op.join(self.collect.static_url, '')): # noqa
                            opath = op.join(
                                op.relpath(
                                    bp.static_url_path,
                                    self.collect.static_url), opath)
                        yield bp, fpath, opath

    def log(self, msg):
        """ Log message. """
        if self.verbose:
            print(msg)



from flask import Flask, Blueprint
from flask_collect import Collect
from functools import partial
from os import path as op
from tempfile import mkdtemp
from unittest import TestCase


class BaseTest(TestCase):

    def test_collect(self):
        app = Flask(__name__)

        blueprint = Blueprint(
            'test1', __name__, static_folder='static1',
            static_url_path='/static/test1')
        app.register_blueprint(blueprint)

        blueprint = Blueprint('test2', __name__, static_folder='static2')
        app.register_blueprint(blueprint)

        static_root = mkdtemp()

        app.config['COLLECT_STATIC_ROOT'] = static_root

        collect = Collect(app)
        collect.collect(verbose=True)

        self.assertTrue(op.exists(op.join(static_root, 'test1', 'test.css')))
        self.assertTrue(op.exists(op.join(static_root, 'js', 'test.js')))
        self.assertTrue(op.exists(op.join(static_root, 'app.css')))

        app.config['COLLECT_STORAGE'] = 'flask.ext.collect.storage.test'
        collect = Collect(app)
        test = collect.collect(verbose=True)
        self.assertEqual(len(test), 3)

    def test_filter(self):
        app = Flask(__name__)

        blueprint = Blueprint('test1', __name__, static_folder='static1')
        app.register_blueprint(blueprint)

        blueprint = Blueprint('test3', __name__, static_folder='static3')
        app.register_blueprint(blueprint)

        static_root = mkdtemp()

        def filter_(order, items):
            def _key(item):
                if item.name in order:
                    return order.index(item.name)
                return -1
            return sorted(items, key=_key)

        app.config['COLLECT_STATIC_ROOT'] = static_root
        app.config['COLLECT_FILTER'] = partial(filter_, ['test3', 'test1'])
        app.config['COLLECT_STORAGE'] = 'flask.ext.collect.storage.test'

        collect = Collect(app)
        test = list(collect.collect(verbose=True))
        self.assertEqual(len(test), 3)
        self.assertTrue('static3' in test[1][1])

        app.config['COLLECT_FILTER'] = partial(filter_, ['test1', 'test3'])
        collect = Collect(app)
        test = list(collect.collect(verbose=True))
        self.assertTrue('static1' in test[1][1])



from os import path as op, makedirs
from shutil import copy

from .base import BaseStorage


class Storage(BaseStorage):

    def run(self):
        self.log("Collect static from blueprints.")
        destination_list = set()
        for bp, f, o in self:
            destination = op.join(self.collect.static_root, o)
            destination_dir = op.dirname(destination)
            if not op.exists(destination_dir):
                makedirs(destination_dir)
            if destination in destination_list:
                self.log("{0} already copied".format(destination))
            if not op.exists(destination) or op.getmtime(destination) < op.getmtime(f):
                copy(f, destination)
                self.log(
                    "Copied: [%s] '%s'" % (bp.name, op.join(self.collect.static_url, destination)))
            destination_list.add(destination)



# -*- coding: utf-8 -*-
"""
    Flask-Collect
    =============

    Flack-Collect is simply application for collect static files in Flask_
    project.
    Serve static files with Flask_ -- bad idea for production, with this you
    will can collect them in one command.

    This extension checks application blueprints for static files and copy it
    to specific folder (saves related paths).

"""

from .collect import Collect # noqa

__author__ = "Kirill Klenov <horneds@gmail.com>"
__license__ = "BSD"
__project__ = __name__
__version__ = "1.0.4"
